# User interface to the libifp driver.
# This should be the only module needed to interface with the iFP
# 
# $Id: ifp.py,v 1.6 2005/02/21 06:35:15 milamber Exp $
#


import usb_core
import ifp_core

import os
import ntpath # iFP devices use \ path separators

# Bring in some constants from the core
IFP_FILE = ifp_core.IFP_FILE
IFP_DIR = ifp_core.IFP_DIR
IFP_ERR_DEV_FUBAR = ifp_core.IFP_ERR_DEV_FUBAR
IFP_ERR_BAD_FREQUENCY = ifp_core.IFP_ERR_BAD_FREQUENCY
IFP_ERR_BAD_FILENAME = ifp_core.IFP_ERR_BAD_FILENAME
IFP_ERR_USER_CANCEL = ifp_core.IFP_ERR_USER_CANCEL
IFP_TUNER_PRESET_DATA = ifp_core.IFP_TUNER_PRESET_DATA
IFP_FREQ_MIN = ifp_core.IFP_FREQ_MIN
IFP_FREQ_MAX = ifp_core.IFP_FREQ_MAX
IFP_PRESET_TOTAL = ifp_core.IFP_PRESET_TOTAL
IFP_TUNER_LABEL = ifp_core.IFP_TUNER_LABEL
IFP_MAX_PATH = ifp_core.IFP_MAX_PATH
IFP_MAX_FILENAME = ifp_core.IFP_MAX_FILENAME
IFP_WALK_FILE = ifp_core.IFP_WALK_FILE
IFP_WALK_DIR_PRE = ifp_core.IFP_WALK_DIR_PRE
IFP_WALK_DIR_POST = ifp_core.IFP_WALK_DIR_POST
IFP_WALK_NONE = ifp_core.IFP_WALK_NONE

class iFP:
    """Class to wrap up the low-level functionality of the ifp_core module around libifp.
    
    After the object is created, init_device() must be called before anything useful can 
    be done.
    """
    def __init__(self):
        """Creates the ifp object. This function does not connect to the iFP
        device to make it easier to catch initilization errors.
        
        You must call init_device before using any of the functions of the ifp
        object except for init_device,close, and isOK.
        """
        self.ifp = None

    def __del__(self):
        """Clean up by closing the device if the user forgot about"""
        self.close()
        
    def init_device(self):
        """Connect to the iFP device. If this function returns witout
        throwing an exception the ifp object should be able to 
        communicate with the iFP device.
        """
        if self.ifp is not None: return
        usb_core.usb_init()
        self.device_handle = ifp_core.ifp_find_device()
        self.device = usb_core.usb_device(self.device_handle)
        num = usb_core.get_dev_interface_number(self.device)
        usb_core.usb_claim_interface(self.device_handle,num)
        self.ifp = ifp_core.malloc_ifp_device()
        ifp_core.ifp_init(self.ifp,self.device_handle)
        
    def get_battery(self):
        """Returns the battery level of the iFP
        
        This seems to be horribly inaccurate, I've only ever seen 1 and 4
        with a very rare 2.
        """
        return ifp_core.ifp_battery(self.ifp)

    def get_capacity(self):
        """Return the capacity of the iFP in bytes"""
        try: return self._capacity
        except AttributeError:
            self._capacity = ifp_core.ifp_capacity(self.ifp)
            return self._capacity
    capacity = property(get_capacity,
                        doc = "device storage capacity in bytes")
    
        
    def get_free_space(self):
        """Return the free space available on the iFP in bytes"""
        return ifp_core.ifp_freespace(self.ifp)
        
    def close(self):
        """Close the iFP device when you are done with it.
        After closing the device no functions other than init_device, close, 
        and isOK can be called on the ifp object without an exception.
        
        I haven't tried re-initing a closed device, but I think it whould work
        """
        if self.ifp is not None:
            ifp_core.ifp_finalize(self.ifp)
            usb_core.usb_release_interface(self.device_handle,usb_core.get_dev_interface_number(self.device))
            ifp_core.ifp_release_device(self.device_handle)
            self.device = None
            self.ifp = None

            try: del(self._capacity)
            except AttributeError: pass
            try: del(self._model)
            except AttributeError: pass
            try: del(self._firmware)
            except AttributeError: pass
            
    def __nonzero__(self):
        """Check to see if the ifp object contains a valid handle to the iFP
        device. Preforms no checking on the device itself
        """
        return bool(self.ifp)
        
    def get_model(self):
        """Get the model string from the iFP device"""
        try: return self._model
        except AttributeError:
            self._model = ifp_core.get_ifp_model(self.ifp)
            return self._model
    model = property(get_model, doc = "device model string")
        
    def get_device_info(self):
        """Return the info string from the iFP device. libifp documentation reccomends 
        putting this string in the about box or somewhere easily accessable to help
        report and diagnose problems
        """
        return ifp_core.get_ifp_device_info(self.ifp)
        
    def get_firmware_version(self):
        """Return the firmware version of the iFP device as string of format X.XX"""
        try: return self._firmware
        except AttributeError:
            i = ifp_core.ifp_firmware_version(self.ifp)
            self._firmware = "%x.%02x" % (i / 0x0100,i % 0x0100)
            return self._firmware
    firmware_version = property(get_firmware_version,
                                doc = "device firmware version string")
        
    def get_preset(self,station):
        """Get the preset station info
        
        Returns a tuple (callsign,frequency) with frequency in MHZ
        """
        (ret,call,freq) = ifp_core.get_station_info(self.ifp,station)
        if ret != 0:
            raise(IOError,"Unable to retrieve station info")
        freq = freq / 100.0
        return (call,freq)
        
    def set_preset(self,station,callsign,freq):
        """Set a preset station to callsign and freq (in MHZ)"""
        freq2 = int(freq * 100)
        ifp_core.set_station_info(self.ifp,station,callsign,freq2)
        
    def format(self):
        """Caution! This function crashes the iFP and returns USB errors!
        Oh yeah, it also formats the iFP removing all your data!
        """
        ifp_core.ifp_format(self.ifp)
        
    def listdir(self,directory,callback = None):
        """Call callback once for every file or directory contained
        in the directory on the device. If the callback is None, the
        list of tuples of file information will be returned instead."""
        files = []
        def append_files(kind, size, name): files.append([name,size,kind])
        callback = callback or append_files
        ifp_core.list_dir(self.ifp,directory,callback)
        if files: return files
        else: return None

    def upload(self, local, remote = None, callback = None):
        """Upload a file from the local filesystem to the iFP device.
        If no remote filename is given, place the file in the root
        directory (\\) with the same base name as the local file.

        If a callback is given, it is called during the upload with
        one argument, the transfer status. If the callback returns
        anything other than False, it signifies an error and the
        upload will be stopped.

        The local file can be a directory (but cannot start with
        \ , ?, or !)."""
        callback = callback or (lambda status: 0)
        remote = remote or os.path.basename(local)
        if not os.path.exists(local):
            raise IOError("%s does not exist locally" % local)
        elif os.path.isdir(local):
            ifp_core.upload_dir_status(self.ifp, local, remote, callback)
        else: # regular file
            ifp_core.upload_file_status(self.ifp, local, remote, callback)
        
    def download(self, remote, local = None, callback = None):
        """Download a file from the iFP device to the local filesystem.
        If no local filename is given, place the file in the current
        directory ith the same base name as the remote file.

        If a callback is given, it is called during the download with
        one argument, the transfer status. If the callback returns
        anything other than False, it signifies and error and the
        download will be stopped.

        The remote file can be a directory."""
        callback = callback or (lambda status: 0)
        local = local or ntpath.basename(remote)
        if not self.exists(remote):
            raise IOError("%s does not exist remotely" % local)
        elif self.isdir(remote):
            ifp_core.download_dir_status(self.ifp, remote, local, callback)
        else: # regular file
            ifp_core.download_file_status(self.ifp, remote, local, callback)
        
    def rename(self,old_path,new_path):
        """Rename a file or directory on the iFP device
        It must be a full path
        """
        ifp_core.ifp_rename(self.ifp,old_path,new_path)

    def unlink(self,filename):
        """Delete the named from from the iFP device
        filename must be a full path
        """
        ifp_core.ifp_delete(self.ifp,filename)
        
    def mkdir(self,dirname):
        """Create a new directory on the iFP device.
        
        dirname must be a full pathname
        """
        ifp_core.ifp_mkdir(self.ifp,dirname)

    def makedirs(self, name):
        """Create a new directory on the iFP device, including any
        parent directories."""
        if self.isdir(name): return
        elif self.exists(name):
            raise IOError("%s exists and is not a directory" % name)
        dirname = ntpath.dirname(dirname)
        if dirname != "" and not self.isdir(dirname):
            self.makedirs(dirname)
        self.mkdir(name)

    def rmdir(self,dirname, recursive = False):
        """Remove a directory. If recursive is true, its contents (if any)
        will also be removed"""
        if recursive: ifp_core.ifp_delete_dir_recursive(self.ifp,dirname)
        else: ifp_core.ifp_rmdir(self.ifp,dirname)
        
    def read_open(self,filename):
        ifp_core.ifp_read_open(self.ifp,filename)
        
    def read_close(self):
        ifp_core.ifp_read_close(self.ifp)
    
    def read_seek(self,abs_position):
        ifp_core.ifp_read_seek(self.ifp,abs_position)
        
    def read_data(self,bytes):
        return ifp_core.ifp_read_data(self.ifp,bytes)
        
    def read_eof(self):
        return ifp_core.ifp_read_eof(self.ifp)
        
    def read_size(self):
        return ifp_core.ifp_read_size(self.ifp)
        
    def write_open(self,filename,filesize):
        """Create a new file with a given size on the iFP device
        
        Data must be put in the file with WriteData
        """
        ifp_core.ifp_write_open(self.ifp,filename,filesize)

    def write_close(self):
        """Close the file currently open for writing"""
        ifp_core.ifp_write_close(self.ifp)
        
    def write_data(self,data):
        """Write data to the currently open file"""
        ifp_core.ifp_write_data(self.ifp,data)
        
    def isfile(self,filename):
        """Return true if the named item is a file, false otherwise"""
        return ifp_core.ifp_is_file(self.ifp,filename) == 1

    def isdir(self,dirname):
        """Return true if the named item is a directory, false otherwise"""
        return ifp_core.ifp_is_dir(self.ifp,dirname) == 1

    def exists(self,filename):
        """Returns true if the named item exists"""
        return ifp_core.ifp_exists(self.ifp,filename) > 0
